# SPDX-License-Identifier: GPL-2.0-or-later

from pathlib import Path

import bpy
from bpy.types import Panel
from bl_ui.utils import PresetPanel

from vse_toolbox.bl_utils import (get_addon_prefs, get_scene_settings, get_strip_settings)
from vse_toolbox.constants import ASSET_PREVIEWS
from vse_toolbox.sequencer_utils import (set_active_strip, get_channel_name, get_strips)


class VSETB_main:
    bl_space_type = "SEQUENCE_EDITOR"
    bl_region_type = "UI"
    bl_category = "VSE ToolBox"
    bl_label = "VSE ToolBox"


class VSETB_PT_main(VSETB_main, Panel):

    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.load_projects', icon='FILE_REFRESH', text='', emboss=False)

    def draw(self, context):
        wm = context.window_manager
        scn = context.scene

        settings = get_scene_settings()
        prefs = get_addon_prefs()
        
        project = settings.active_project

        layout = self.layout
        col = layout.column()

        col.prop(settings, 'project_name', text='Project')

        if project:
            if project.type == 'TVSHOW':
                col.prop(project, 'episode_name', text='Episodes')

        #col.separator()

        #row = col.row(align=True)
        
        #row.prop(settings, 'toogle_prefs', text='', icon='PREFERENCES', toggle=True)
        
        '''
        if settings.toogle_prefs:
            box = col.box()
            col = box.column(align=True)
            col.use_property_split = True
            col.use_property_decorate = False
            
            col.prop(settings, 'project_name', text='Projects')

            if project:
                if project.type == 'TV Shows':
                    col.prop(project, 'episode_name', text='Episodes')
                
                #col.prop(project, 'sequence_template')
                #col.prop(project, 'shot_template')
                # col.separator()
                # col.operator('vse_toolbox.new_episode', text='Add Episode', icon='IMPORT')
        '''

        # Rename

class VSETB_PT_strip(Panel):
    bl_space_type = "SEQUENCE_EDITOR"
    bl_region_type = "UI"
    bl_category = "Strip"
    bl_label = "VSE ToolBox"
    #bl_order = 0

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_strip_settings()

        layout.use_property_split = True
        layout.use_property_decorate = False

        layout.prop(settings, 'source_name', text='Source Name')
                     

class VSETB_PT_sequencer(VSETB_main, Panel):
    bl_label = "Sequencer"
    bl_parent_id = "VSETB_PT_main"
    
    def draw_header_preset(self, context):
        settings = get_scene_settings()

        audio_strips = get_strips('Audio')

        depress = any(s.show_waveform for s in audio_strips)
        self.layout.operator('vse_toolbox.show_waveform', text="", icon="IPO_ELASTIC", depress=depress).enabled = not depress

        ico = ("RESTRICT_SELECT_OFF" if settings.auto_select_strip else "RESTRICT_SELECT_ON")
        self.layout.prop(settings, "auto_select_strip", text="", icon=ico)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()
        col.operator('vse_toolbox.set_sequencer', text='Set-Up Sequencer', icon='SEQ_SEQUENCER')

        #row = col.row()
        shot_label = ''
        sequence_label = ''

        if project:
            episode = project.episode_name
            sequence_label = project.sequence_template.format(episode=episode, index=project.sequence_start_number)
            shot_label = project.shot_template.format(episode=episode, sequence=sequence_label, index=project.shot_start_number)
        
        #row.separator()

        strip = context.active_sequence_strip
        channel_name = get_channel_name(strip) or ''
        if channel_name == 'Shots':
            label = shot_label
        elif channel_name == 'Sequences':
            label = sequence_label
        else:
            label = 'Not Supported'

        row = col.row(align=True)
        if label == 'Not Supported':
            row.enabled = False

        op = row.operator('vse_toolbox.strips_rename', text=f'Rename {channel_name}  ( {label} )', icon='SORTALPHA')
        op.channel_name = channel_name

        col.operator('vse_toolbox.set_stamps', text='Set Stamps', icon='COLOR')


class VSETB_PT_settings(VSETB_main, Panel):
    bl_label = "Settings"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    #def draw_header_preset(self, context):
    #    self.layout.operator('vse_toolbox.import_files', icon='IMPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()
        #row = col.row(align=True)
        col.prop(project, 'sequence_template')
        col.prop(project, 'shot_template')
        col.prop(project, 'render_template')


class VSETB_PT_imports(VSETB_main, Panel):
    bl_label = "Imports"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.import_files', icon='IMPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()
        #row = col.row(align=True)
        #col.operator('vse_toolbox.import_files', text='Import', icon='IMPORT')
        col.operator('vse_toolbox.import_spreadsheet', text='Import Spreadsheet', icon='SPREADSHEET')


class VSETB_PT_presets(PresetPanel, Panel):
    bl_label = 'Spreadsheet Presets'
    preset_subdir = 'vse_toolbox'
    preset_operator = 'script.execute_preset'
    preset_add_operator = "vse_toolbox.add_spreadsheet_preset"


class VSETB_PT_exports(VSETB_main, Panel):
    bl_label = "Exports"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.export_spreadsheet', icon='EXPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        # TODO FAIRE DES VRAIS OPS
        layout.operator('vse_toolbox.strips_render', text='Render Strips', icon='SEQUENCE')

        tracker_label = settings.tracker_name.title().replace('_', ' ')
        layout.operator('vse_toolbox.upload_to_tracker', text=f'Upload to {tracker_label}', icon='EXPORT')
        layout.operator('vse_toolbox.export_spreadsheet', text='Export Spreadsheet', icon='SPREADSHEET')


class VSETB_PT_casting(VSETB_main, Panel):
    bl_label = "Casting"
    bl_parent_id = "VSETB_PT_main"

    def draw_header_preset(self, context):
        active_strip = context.scene.sequence_editor.active_strip
        self.layout.label(text=active_strip.name)

    @classmethod
    def poll(cls, context):
        strip = context.scene.sequence_editor.active_strip
        return strip and get_channel_name(strip) == 'Shots'

    def draw(self, context):
        layout = self.layout
        
        settings = get_scene_settings()
        strip_settings = get_strip_settings()

        project = settings.active_project

        if not project:
            return

        if not project.assets:
            row = layout.row(align=True)
            row.label(text='No Assets in this Project')
        else:

            row = layout.row()
            col = row.column()
            col.template_list("VSETB_UL_casting", "shot_casting", strip_settings, "casting", strip_settings, "casting_index", rows=6)
                                
            col_tool = row.column(align=True)
            col_tool.operator('vse_toolbox.casting_add', icon='ADD', text="")
            col_tool.operator('vse_toolbox.casting_remove', icon='REMOVE', text="")
            col_tool.separator()
            col_tool.operator('vse_toolbox.casting_move', icon='TRIA_UP', text="").direction = 'UP'
            col_tool.operator('vse_toolbox.casting_move', icon='TRIA_DOWN', text="").direction = 'DOWN'
            col_tool.separator()
            col_tool.operator('vse_toolbox.copy_casting', icon='COPYDOWN', text="")
            col_tool.operator('vse_toolbox.paste_casting', icon='PASTEDOWN', text="")
            col_tool.separator()

            col_tool.operator('vse_toolbox.casting_replace', icon='ZOOM_ALL', text="")

            if strip_settings.casting:
                casting_item = strip_settings.casting[strip_settings.casting_index]
                asset = casting_item.asset
                if asset:        
                    if asset.icon_id:
                        row = col.row(align=True)
                        #row.scale_y = 0.5
                        #     box = col.box()
                        #     box.template_icon(icon_value=ico.icon_id, scale=7.5)

                        row.template_icon_view(asset, "previews", show_labels=False)


class VSETB_PT_metadata(VSETB_main, Panel):
    bl_label = "Shot Metadata"
    bl_parent_id = "VSETB_PT_casting"
    #bl_options = {"DEFAULT_CLOSED"}

    def draw(self, context):
        layout = self.layout
        scn = context.scene
        active_strip = scn.sequence_editor.active_strip
        
        if not active_strip:
            return 

        settings = get_scene_settings()
        strip_settings = get_strip_settings()

        project = settings.active_project

        if not project:
            return
        
        col = layout.column(align=True)
        col.prop(strip_settings, 'description', text='DESCRIPTION')

        for metadata_type in project.metadata_types:
            if metadata_type.entity_type == 'SHOT':
                #row = layout.row(align=False)
                metadata_key = metadata_type.field_name
                metadata_label = metadata_key.upper()

                if metadata_type.choices:
                    metadata_value = getattr(strip_settings.metadata, metadata_key)
                    icon = 'LAYER_USED'
                    if metadata_value:
                        if metadata_value in metadata_type.choices:
                            icon = 'DOT'
                        else:
                            icon = 'ADD'

                    col.prop_search(strip_settings.metadata, metadata_key, metadata_type, 'choices', 
                        results_are_suggestions=True, icon=icon, text=metadata_label)
                    
                else:
                    col.prop(strip_settings.metadata, metadata_key, text=metadata_label)

                #row.operator('vse_toolbox.copy_metadata', icon='PASTEDOWN', text='', emboss=False).metadata = metadata_key

def context_menu_prop(self, context):
    if not hasattr(context, 'button_prop') or context.space_data.type != 'SEQUENCE_EDITOR':
        return
    
    settings = get_strip_settings()
    if not settings:
        return

    button_prop = context.button_prop
    if button_prop not in settings.metadata.bl_rna.properties.values():
        return
    
    layout = self.layout
    layout.separator()
    layout.operator('vse_toolbox.copy_metadata', icon='PASTEDOWN', text='Copy metadata to selected').metadata = button_prop.name


classes = (
    VSETB_PT_main,
    VSETB_PT_imports,
    VSETB_PT_sequencer,
    VSETB_PT_casting,
    VSETB_PT_metadata,
    VSETB_PT_presets,
    VSETB_PT_exports,
    VSETB_PT_strip
)

def register(): 
    for cls in classes:
        bpy.utils.register_class(cls)
    
    bpy.types.UI_MT_button_context_menu.append(context_menu_prop)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    
    bpy.types.UI_MT_button_context_menu.remove(context_menu_prop)